@@ -551,14 +551,15 @@ void say_there_is_no_help(void)
 }
 #endif
 
-/* Make nano exit gracefully. */
+/* Exit normally: restore the terminal state and save history files. */
 void finish(void)
 {
-	/* Blank the statusbar and (if applicable) the shortcut list,
-	 * and move the cursor to the last line of the screen. */
+	/* Blank the statusbar and (if applicable) the shortcut list. */
 	blank_statusbar();
 	blank_bottombars();
 	wrefresh(bottomwin);
+
+	/* Switch on the cursor and exit from curses mode. */
 	curs_set(1);
 	endwin();
 
@@ -582,7 +583,8 @@ void finish(void)
 	exit(0);
 }
 
-/* Make nano die gracefully. */
+/* Die gracefully -- by restoring the terminal state and saving any buffers
+ * that were modified. */
 void die(const char *msg, ...)
 {
 	va_list ap;
@@ -888,6 +890,9 @@ void usage(void)
 	print_opt("-w", "--nowrap", N_("Don't hard-wrap long lines"));
 #endif
 	print_opt("-x", "--nohelp", N_("Don't show the two help lines"));
+#ifndef NANO_TINY
+	print_opt("-y", "--afterends", N_("Make Ctrl+Right stop at word ends"));
+#endif
 	if (!ISSET(RESTRICTED))
 		print_opt("-z", "--suspend", N_("Enable suspension"));
 #ifndef NANO_TINY
@@ -1107,7 +1112,7 @@ bool scoop_stdin(void)
 	endwin();
 	tcsetattr(0, TCSANOW, &oldterm);
 
-	fprintf(stderr, _("Reading from stdin, ^C to abort\n"));
+	fprintf(stderr, _("Reading from standard input; type ^D or ^D^D to finish.\n"));
 
 #ifndef NANO_TINY
 	/* Enable interpretation of the special control keys so that
@@ -1142,6 +1147,7 @@ bool scoop_stdin(void)
 	open_buffer("", TRUE);
 	read_file(stream, 0, "stdin", TRUE);
 	openfile->edittop = openfile->fileage;
+	fprintf(stderr, ".\n");
 
 	/* Reconnect the tty as the input source. */
 	thetty = open("/dev/tty", O_RDONLY);
@@ -1210,6 +1216,18 @@ void signal_init(void)
 		sigaction(SIGTSTP, &act, NULL);
 #endif
 	}
+
+#if !defined(NANO_TINY) && !defined(DEBUG)
+	if (getenv("NANO_NOCATCH") == NULL) {
+		/* Trap SIGSEGV and SIGABRT to save any changed buffers and reset
+		 * the terminal to a usable state.  Reset these handlers to their
+		 * defaults as soon as their signal fires. */
+		act.sa_handler = handle_crash;
+		act.sa_flags |= SA_RESETHAND;
+		sigaction(SIGSEGV, &act, NULL);
+		sigaction(SIGABRT, &act, NULL);
+	}
+#endif
 }
 
 /* Handler for SIGHUP (hangup) and SIGTERM (terminate). */
@@ -1218,6 +1236,14 @@ RETSIGTYPE handle_hupterm(int signal)
 	die(_("Received SIGHUP or SIGTERM\n"));
 }
 
+#if !defined(NANO_TINY) && !defined(DEBUG)
+/* Handler for SIGSEGV (segfault) and SIGABRT (abort). */
+RETSIGTYPE handle_crash(int signal)
+{
+	die(_("Sorry! Nano crashed!  Code: %d.  Please report a bug.\n"), signal);
+}
+#endif
+
 /* Handler for SIGTSTP (suspend). */
 RETSIGTYPE do_suspend(int signal)
 {
@@ -1982,6 +2008,7 @@ int main(int argc, char **argv)
 		{"autoindent", 0, NULL, 'i'},
 		{"cutfromcursor", 0, NULL, 'k'},
 		{"unix", 0, NULL, 'u'},
+		{"afterends", 0, NULL, 'y'},
 		{"softwrap", 0, NULL, '$'},
 #endif
 		{NULL, 0, NULL, 0}
@@ -2040,15 +2067,9 @@ int main(int argc, char **argv)
 
 	while ((optchr =
 		getopt_long(argc, argv,
-				"ABC:DEFGHIKLMNOPQ:RST:UVWX:Y:abcdefghijklmno:pqr:s:tuvwxz$",
+				"ABC:DEFGHIKLMNOPQ:RST:UVWX:Y:abcdefghijklmno:pqr:s:tuvwxyz$",
 				long_options, NULL)) != -1) {
 		switch (optchr) {
-			case 'b':
-			case 'e':
-			case 'f':
-			case 'j':
-				/* Pico compatibility flags. */
-				break;
 #ifndef NANO_TINY
 			case 'A':
 				SET(SMART_HOME);
@@ -2165,6 +2186,9 @@ int main(int argc, char **argv)
 			case 'g':
 				SET(SHOW_CURSOR);
 				break;
+			case 'h':
+				usage();
+				exit(0);
 #ifndef NANO_TINY
 			case 'i':
 				SET(AUTOINDENT);
@@ -2173,6 +2197,11 @@ int main(int argc, char **argv)
 				SET(CUT_FROM_CURSOR);
 				break;
 #endif
+#ifdef ENABLE_LINENUMBERS
+			case 'l':
+				SET(LINE_NUMBERS);
+				break;
+#endif
 #ifdef ENABLE_MOUSE
 			case 'm':
 				SET(USE_MOUSE);
@@ -2235,6 +2264,11 @@ int main(int argc, char **argv)
 			case 'x':
 				SET(NO_HELP);
 				break;
+#ifndef NANO_TINY
+			case 'y':
+				SET(AFTER_ENDS);
+				break;
+#endif
 			case 'z':
 				SET(SUSPEND);
 				break;
@@ -2243,14 +2277,11 @@ int main(int argc, char **argv)
 				SET(SOFTWRAP);
 				break;
 #endif
-#ifdef ENABLE_LINENUMBERS
-			case 'l':
-				SET(LINE_NUMBERS);
+			case 'b':  /* Pico compatibility flags. */
+			case 'e':
+			case 'f':
+			case 'j':
 				break;
-#endif
-			case 'h':
-				usage();
-				exit(0);
 			default:
 				printf(_("Type '%s -h' for a list of available options.\n"), argv[0]);
 				exit(1);
@@ -2649,6 +2680,7 @@ int main(int argc, char **argv)
 		} else
 			edit_refresh();
 
+		errno = 0;
 		focusing = TRUE;
 
 		/* Forget any earlier statusbar x position. */
